import json
import re
import socketserver
from dataclasses import asdict
from http.server import BaseHTTPRequestHandler, HTTPServer

from Code.Communication.Direction import Direction
from Code.Communication.Neighbours import Neighbours
from Code.UI.Field import Field
from Code.UI.PlayingField import GameState


class RequestHandler(BaseHTTPRequestHandler):
	neighbours: Neighbours = None
	game_state: GameState = None

	def do_GET(self):
		print("got Get request")
		if self.path == "/":
			self.send_response(200, "running")
		elif self.path.startswith("/border/"):
			direction = re.findall("/border/(left|right)", self.path, flags=re.IGNORECASE)[0]
			cells = self.game_state.field.get_edge(Direction[direction.upper()])

			self.send_response(200)
			self.send_header('Content-Type', 'application/json')
			self.end_headers()
			self.wfile.write(json.dumps(cells).encode('utf8'))

	def handle(self) -> None:
		super().handle()


	"""
	/connect/right
	/connect/left
	
	with body : {ip:"string",port:number}
	"""

	def do_POST(self):
		if self.path.startswith("/connect/"):
			direction = re.findall("/connect/(left|right)", self.path, flags=re.IGNORECASE)[0]
			data_string = self.rfile.read(int(self.headers['Content-Length']))
			print(f"Got conenction request with value {data_string}")
			data = json.loads(data_string)
			neighbour, accepted = self.neighbours.accept_connection(Direction[direction.upper()], data["ip"], data["port"])

			print(f"Sending neighbour: {neighbour}")
			if accepted:
				self.send_response(200)
				self.send_header('Content-Type', 'application/json')
				self.end_headers()
				self.wfile.write(json.dumps(asdict(neighbour)).encode('utf8'))
			else:
				self.send_response(303)
				self.send_header('Location', f"http://{neighbour.ip}:{neighbour.port}")
				self.end_headers()

		self.end_headers()

